@@ -1,4 +1,6 @@
-use core::{cmp::Reverse, fmt::Display, hash::Hash};
+//! Basic battle host
+
+use core::{cmp::Reverse, fmt::Display, hash::Hash, cell::{RefMut}};
 use log::{info, warn};
 use rand::Rng;
 
@@ -10,48 +12,45 @@ use pokedex::{
 };
 
 use crate::{
-    data::*,
-    message::{ClientMessage, EndState, ServerMessage},
-    moves::{damage::DamageResult, BattleMove, ClientMove, ClientMoveAction},
-    party::PartyIndex,
-    player::{LocalPlayer, ValidatedPlayer},
-    pokemon::PokemonIdentifier,
-    BattleEndpoint, Indexed,
+    data::*, 
+    endpoint::ReceiveError, 
+    engine::{MoveEngine, MoveResult}, 
+    message::{ClientMessage, ServerMessage, TimedAction, StartableAction, FailedAction}, 
+    moves::{BattleMove, ClientMove, ClientMoveAction, damage::{ClientDamage, DamageResult}}, 
+    player::{ClientPlayerData}, 
+    pokemon::{Indexed, PokemonIdentifier}
 };
 
 mod collections;
 mod party;
 mod pokemon;
-
-pub mod engine;
-pub mod player;
+mod timer;
+mod player;
 
 use collections::BattleMap;
-use engine::{MoveEngine, MoveResult};
-use player::BattlePlayer;
+use player::{BattlePlayer, PlayerData};
 use party::BattleParty;
-use pokemon::BattlePokemon;
+use pokemon::HostPokemon;
 use std::collections::BTreeMap;
+use timer::*;
 
 pub mod prelude {
 
-    #[cfg(feature = "default_engine")]
-    pub use super::engine::default::*;
-
-    pub use super::engine::{MoveEngine, MoveResult};
     pub use super::Battle;
+    pub use super::player::PlayerData;
 
 }
 
+/// A battle host.
 pub struct Battle<
     'd,
     ID: Display + Clone + Ord + Hash + 'static,
-    E: BattleEndpoint<ID, AS>,
     const AS: usize,
 > {
     state: BattleState,
     data: BattleData,
-    players: BattleMap<ID, BattlePlayer<'d, ID, E, AS>>, // can change to dex implementation, and impl Identifiable for BattlePlayer
+    players: BattleMap<ID, BattlePlayer<'d, ID, AS>>, // can change to dex implementation, and impl Identifiable for BattlePlayer
+    timer: Timer,
 }
 
 #[derive(Debug)]
@@ -59,8 +58,8 @@ enum BattleState {
     Start,
     StartSelecting,
     WaitSelecting,
-    StartMoving,
-    WaitMoving,
+    QueueMoves,
+    WaitReplace,
     End,
 }
 
@@ -79,38 +78,35 @@ pub enum MovePriority<ID: Ord> {
 impl<
         'd,
         ID: Display + Clone + Ord + Hash + 'static,
-        E: BattleEndpoint<ID, AS>,
         const AS: usize,
-    > Battle<'d, ID, E, AS>
+    > Battle<'d, ID, AS>
 {
     pub fn new<R: Rng>(
         data: BattleData,
         random: &mut R,
-        pokedex: &'d impl Dex<Pokemon>,
-        movedex: &'d impl Dex<Move>,
-        itemdex: &'d impl Dex<Item>,
-        players: impl IntoIterator<Item = LocalPlayer<ID, E, AS>>,
+        pokedex: &'d dyn Dex<Pokemon>,
+        movedex: &'d dyn Dex<Move>,
+        itemdex: &'d dyn Dex<Item>,
+        players: impl Iterator<Item = PlayerData<ID, AS>>,
     ) -> Self {
+
         Self {
+            players: players
+            .map(|p| {
+                let p = p.init(random, pokedex, movedex, itemdex);
+                (p.id().clone(), p)
+            })
+            .collect(),
             state: BattleState::default(),
             data,
-            players: players
-                .into_iter()
-                .map(|p| {
-                    let p = p.init(random, pokedex, movedex, itemdex);
-                    (p.id().clone(), p)
-                })
-                .collect(),
+            timer: Default::default(),
         }
     }
 
     pub fn begin(&mut self) {
-        for mut player in self.players.values_mut() {
-            player.party.reveal_active();
-        }
 
         for mut player in self.players.values_mut() {
-            let v = ValidatedPlayer::new(self.data.clone(), &player, self.players.values());
+            let v = ClientPlayerData::new(self.data.clone(), &player, self.players.values());
             player.send(ServerMessage::Begin(v));
         }
 
@@ -121,8 +117,8 @@ impl<
         self.state = BattleState::End;
         for (id, mut player) in self.players.iter_mut() {
             match Some(id) == winner.as_ref() {
-                true => player.send(ServerMessage::End(EndState::Win)),
-                false => player.send(ServerMessage::End(EndState::Lose)),
+                true => player.send(ServerMessage::End),
+                false => player.send(ServerMessage::End),
             }
         }
     }
@@ -133,99 +129,36 @@ impl<
         engine: &mut ENG,
         itemdex: &'d impl Dex<Item>,
     ) {
-        for (waiting, mut player) in self.players.values_waiting_mut() {
-            while let Some(message) = player.receive() {
-                match message {
-                    ClientMessage::Move(active, bmove) => {
-                        match player
-                            .party
-                            .active
-                            .get_mut(active)
-                            .map(Option::as_mut)
-                            .flatten()
-                        {
-                            Some(pokemon) => {
-                                pokemon.queued_move = Some(bmove);
-                                // party.client.confirm_move(active);
-                            }
-                            None => warn!(
-                                "Party {} could not add move #{} to pokemon #{}",
-                                player.party.name(),
-                                bmove,
-                                active
-                            ),
-                        }
-                    }
-                    ClientMessage::ReplaceFaint(active, index) => {
-                        match player.party.active_contains(index) {
-                            false => match player.party.pokemon.get(index) {
-                                Some(pokemon) => match pokemon.fainted() {
-                                    false => {
-                                        player.party.active[active] = Some(index.into());
-                                        let unknown = player.party.know(index);
-                                        for mut other in self.players.values_mut() {
-                                            if let Some(pokemon) = unknown.as_ref() {
-                                                other.send(ServerMessage::AddRemote(
-                                                    PokemonIdentifier(
-                                                        player.party.id().clone(),
-                                                        index,
-                                                    ),
-                                                    pokemon.clone(),
-                                                ));
-                                            }
-                                            other.send(ServerMessage::FaintReplace(
-                                                PokemonIdentifier(
-                                                    player.party.id().clone(),
-                                                    active,
-                                                ),
-                                                index,
-                                            ));
-                                        }
-
-                                        player
-                                            .send(ServerMessage::ConfirmFaintReplace(active, true));
-                                    }
-                                    true => player
-                                        .send(ServerMessage::ConfirmFaintReplace(active, false)),
-                                },
-                                None => {
-                                    player.send(ServerMessage::ConfirmFaintReplace(active, false))
-                                }
-                            },
-                            true => player.send(ServerMessage::ConfirmFaintReplace(active, false)),
-                        }
-                    }
-                    ClientMessage::Forfeit => {
-                        // self.remove(player.party.id);
-                        // Self::set_winner(Some(other.party.id), player, other),
-                    }
-                    ClientMessage::LearnMove(pokemon, move_id, index) => {
-                        if let Some(pokemon) = player.party.pokemon.get_mut(pokemon) {
-                            if pokemon.learnable_moves.remove(&move_id) {
-                                pokemon.moves.add(Some(index as _), &move_id);
-                            }
-                        }
-                    }
-                    ClientMessage::FinishedTurnQueue => waiting.set(true),
-                }
-            }
+        for player in self.players.values_mut() {
+            self.receive(player);
         }
 
+        self.update_state(random, engine, itemdex);
+
+    }
+
+    fn update_state<R: Rng + Clone + 'static, ENG: MoveEngine>(&mut self,
+        random: &mut R,
+        engine: &mut ENG,
+        itemdex: &'d impl Dex<Item>,) {
         match self.state {
             BattleState::Start => self.begin(),
             BattleState::StartSelecting => {
-                for (waiting, mut player) in self.players.values_waiting_mut() {
-                    waiting.set(false);
-                    player.send(ServerMessage::StartSelecting);
+                for mut player in self.players.values_mut() {
+                    player.send(ServerMessage::Start(StartableAction::Selecting));
                 }
                 self.state = BattleState::WaitSelecting;
             }
             BattleState::WaitSelecting => {
                 if self.players.values().all(|p| p.party.ready_to_move()) {
-                    self.state = BattleState::StartMoving;
+                    self.state = BattleState::QueueMoves;
+                } else if self.timer.wait(TimedAction::Selecting) {
+                    for mut player in self.players.values_mut().filter(|p| !p.party.ready_to_move()) {
+                        player.send(ServerMessage::Ping(TimedAction::Selecting));
+                    }
                 }
             }
-            BattleState::StartMoving => {
+            BattleState::QueueMoves => {
                 let queue = move_queue(&mut self.players, random);
 
                 let player_queue = self.client_queue(random, engine, itemdex, queue);
@@ -233,25 +166,31 @@ impl<
                 // end queue calculations
 
                 for mut player in self.players.values_mut() {
-                    player.send(ServerMessage::TurnQueue(player_queue.clone()));
+                    player.send(ServerMessage::Start(StartableAction::Turns(player_queue.clone())));
                 }
 
-                self.state = BattleState::WaitMoving;
-            }
-            BattleState::WaitMoving => {
-                if self.players.all_waiting() {
-                    for (id, player) in self.players.iter() {
-                        if player.party.all_fainted() {
-                            self.remove_player(id, EndState::Lose);
-                        }
+                for (id, player) in self.players.iter() {
+                    if player.party.all_fainted() {
+                        self.remove_player(id);
                     }
+                }
 
-                    if self.players.active() <= 1 {
-                        let winner = self.players.keys().next().cloned();
-                        self.end(winner);
-                        return;
-                    } else if self.players.values().all(|p| !p.party.needs_replace()) {
-                        self.state = BattleState::StartSelecting;
+                if self.players.active() <= 1 {
+                    let winner = self.players.keys().next().cloned();
+                    self.end(winner);
+                    return;
+                }
+
+                self.state = BattleState::WaitReplace;
+                self.update_state(random, engine, itemdex);
+            }
+            BattleState::WaitReplace => {
+                match self.players.values().all(|p| !p.party.needs_replace()) {
+                    true => self.state = BattleState::StartSelecting,
+                    false => if self.timer.wait(TimedAction::Replace) {
+                        for mut player in self.players.values_mut().filter(|p| p.party.needs_replace()) {
+                            player.send(ServerMessage::Ping(TimedAction::Replace));
+                        }
                     }
                 }
             }
@@ -259,9 +198,9 @@ impl<
         }
     }
 
-    pub fn remove_player(&self, id: &ID, kind: EndState) {
+    pub fn remove_player(&self, id: &ID) {
         if let Some(mut player) = self.players.deactivate(id) {
-            player.send(ServerMessage::End(kind));
+            player.send(ServerMessage::End);
         }
     }
 
@@ -280,7 +219,105 @@ impl<
         }
     }
 
-    pub fn client_queue<R: Rng + Clone + 'static, ENG: MoveEngine>(
+    fn receive(&self, mut player: RefMut<BattlePlayer<ID, AS>>) {
+        loop {
+            match player.receive() {
+                Ok(message) => match message {
+                    ClientMessage::Move(active, bmove) => {
+                        if match &bmove {
+                            BattleMove::Move(index, ..) => {
+                                // maybe add check for incorrect targeting here?
+                                if !player.party.active(active).map(|p| p.moves.len() > *index).unwrap_or_default() {
+                                    player.send(ServerMessage::Fail(FailedAction::Move(active)));
+                                    false
+                                } else {
+                                    true
+                                }
+                            },
+                            BattleMove::Switch(to) => {
+                                if player.party.active_contains(*to) {
+                                    player.send(ServerMessage::Fail(FailedAction::Switch(active)));
+                                    false
+                                } else {
+                                    true
+                                }
+                            },
+                            BattleMove::UseItem(..) => true,
+                        } {
+                            match player
+                                .party
+                                .active
+                                .get_mut(active)
+                                .map(Option::as_mut)
+                                .flatten()
+                            {
+                                Some(pokemon) => pokemon.queued_move = Some(bmove),
+                                None => warn!(
+                                    "Party {} could not add move #{} to pokemon #{}",
+                                    player.party.name(),
+                                    bmove,
+                                    active
+                                ),
+                            }
+                        }
+                    }
+                    ClientMessage::ReplaceFaint(active, index) => {
+                        if match player.party.active_contains(index) {
+                            false => match player.party.pokemon.get(index) {
+                                Some(pokemon) => match pokemon.fainted() {
+                                    false => {
+                                        player.party.active[active] = Some(index.into());
+                                        let unknown = player.party.know(index);
+                                        for mut other in self.players.values_mut() {
+                                            let id = PokemonIdentifier(
+                                                player.party.id().clone(),
+                                                index,
+                                            );
+                                            if let Some(pokemon) = unknown.as_ref() {
+                                                other.send(ServerMessage::AddRemote(Indexed(
+                                                    id.clone(),
+                                                    pokemon.clone(),
+                                                )));
+                                            }
+                                            other.send(ServerMessage::Replace(Indexed(
+                                                PokemonIdentifier(id.team().clone(), index),
+                                                id.index(),
+                                            )));
+                                        }
+                                        false
+                                    }
+                                    true => true,
+                                },
+                                None => true,
+                            },
+                            true => true,
+                        } {
+                            player
+                                        .send(ServerMessage::Fail(FailedAction::Replace(active)));
+                        }
+                    }
+                    ClientMessage::Forfeit => {
+                        self.remove_player(player.id());
+                    }
+                    ClientMessage::LearnMove(pokemon, move_id, index) => {
+                        if let Some(pokemon) = player.party.pokemon.get_mut(pokemon) {
+                            if pokemon.learnable_moves.remove(&move_id) {
+                                pokemon.moves.add(Some(index as _), &move_id);
+                            }
+                        }
+                    }
+                },
+                Err(err) => match err {
+                    Some(err) => match err {
+                        ReceiveError::Disconnected => self.remove_player(player.id()),
+                    }
+                    None => break,
+                }
+            }
+        }
+    }
+
+    fn client_queue<R: Rng + Clone + 'static, ENG: MoveEngine>(
         &mut self,
         random: &mut R,
         engine: &mut ENG,
@@ -330,7 +367,7 @@ impl<
                                             /// calculates hp and adds it to actions
                                             fn on_damage<'d, ID>(
                                                 location: PokemonIdentifier<ID>,
-                                                pokemon: &mut BattlePokemon<'d>,
+                                                pokemon: &mut HostPokemon<'d>,
                                                 actions: &mut Vec<Indexed<ID, ClientMoveAction>>,
                                                 result: DamageResult<Health>,
                                             ) {
@@ -338,11 +375,11 @@ impl<
                                                     pokemon.hp.saturating_sub(result.damage);
                                                 actions.push(Indexed(
                                                     location,
-                                                    ClientMoveAction::SetDamage(DamageResult {
+                                                    ClientMoveAction::SetHP(ClientDamage::Result(DamageResult {
                                                         damage: pokemon.percent_hp(),
                                                         effective: result.effective,
                                                         crit: result.crit,
-                                                    }),
+                                                    })),
                                                 ));
                                             }
 
@@ -370,9 +407,9 @@ impl<
                                                     };
                                                     actions.push(Indexed(
                                                         target_id,
-                                                        ClientMoveAction::SetHP(
+                                                        ClientMoveAction::SetHP(ClientDamage::Number(
                                                             target.percent_hp(),
-                                                        ),
+                                                        )),
                                                     ));
                                                 }
                                                 MoveResult::Stat(stat, stage) => {
@@ -382,7 +419,10 @@ impl<
                                                         ClientMoveAction::AddStat(stat, stage),
                                                     ));
                                                 }
-                                                MoveResult::Flinch => target.flinch = true,
+                                                MoveResult::Flinch => actions.push(Indexed(
+                                                    target_id,
+                                                    ClientMoveAction::Flinch,
+                                                )),
                                                 MoveResult::Miss => {
                                                     actions.push(Indexed(
                                                         target_id,
@@ -416,7 +456,7 @@ impl<
                                                             .unwrap();
 
                                                         user.learnable_moves.extend(
-                                                            user.instance.add_exp(experience),
+                                                            user.p.add_exp(experience),
                                                         );
 
                                                         actions.push(Indexed(
@@ -455,7 +495,7 @@ impl<
                         ));
                     }
                 }
-                BattleMove::UseItem(id, target) => match itemdex.try_get(&id) {
+                BattleMove::UseItem(Indexed(target, id)) => match itemdex.try_get(&id) {
                     Some(item) => {
                         let mut user = match self.players.get_mut(user_id.team()) {
                             Some(p) => p,
@@ -471,27 +511,11 @@ impl<
                             }
                             ItemUsageKind::Pokeball => match self.data.type_ {
                                 BattleType::Wild => {
-                                    if let Some(mut other) = self.players.get_mut(target.team()) {
-                                        if let Some(active) = other
-                                            .party
-                                            .active
-                                            .get_mut(target.index())
-                                            .map(Option::take)
-                                            .flatten()
-                                        {
-                                            if let Some(pokemon) = other.party.take(active.index())
-                                            {
-                                                user.send(ServerMessage::Catch(
-                                                    pokemon.instance.uninit(),
-                                                ));
-
-                                                true
-                                            } else {
-                                                false
-                                            }
-                                        } else {
-                                            false
-                                        }
+                                    if let Some(pokemon) = self.players.get_mut(target.team()).map(|mut other| other.party.take(target.index())).flatten() {
+                                        user.send(ServerMessage::Catch(
+                                            pokemon.p.p.uninit(),
+                                        ));
+                                        true
                                     } else {
                                         false
                                     }
@@ -505,7 +529,7 @@ impl<
                         } {
                             player_queue.push(Indexed(
                                 user_id,
-                                ClientMove::UseItem(item.id, target.index()),
+                                ClientMove::UseItem(Indexed(target, item.id)),
                             ));
                         }
                     }
@@ -522,10 +546,10 @@ impl<
                             .flatten()
                         {
                             for mut other in self.players.values_mut() {
-                                other.send(ServerMessage::AddRemote(
+                                other.send(ServerMessage::AddRemote(Indexed(
                                     PokemonIdentifier(user.party.id().clone(), new),
                                     unknown.clone(),
-                                ))
+                                )))
                             }
                         }
                         player_queue.push(Indexed(user_id, ClientMove::Switch(new)));
@@ -538,8 +562,8 @@ impl<
     }
 }
 
-pub fn move_queue<ID: Clone + Ord + Hash, E: BattleEndpoint<ID, AS>, R: Rng, const AS: usize>(
-    players: &mut BattleMap<ID, BattlePlayer<ID, E, AS>>,
+pub fn move_queue<ID: Clone + Ord + Hash, R: Rng, const AS: usize>(
+    players: &mut BattleMap<ID, BattlePlayer<ID, AS>>,
     random: &mut R,
 ) -> Vec<Indexed<ID, BattleMove<ID>>> {
     let mut queue = BTreeMap::new();
