@@ -1,39 +1,26 @@
 use pokedex::Uninitializable;
 
 use crate::{
-    party::{PartyIndex, PlayerParty, RemoteParty},
-    pokemon::{
-        remote::{RemotePokemon, UnknownPokemon},
-        ActivePokemon,
-    },
+    party::{PlayerParty, RemoteParty},
+    pokemon::remote::{RemotePokemon, UnknownPokemon},
 };
 
-use super::pokemon::BattlePokemon;
+use super::pokemon::{ActiveBattlePokemon, HostPokemon};
 
 pub type BattleParty<'d, ID, const AS: usize> =
-    PlayerParty<ID, ActivePokemon<ID>, BattlePokemon<'d>, AS>;
+    PlayerParty<ID, ActiveBattlePokemon<ID>, HostPokemon<'d>, AS>;
 
 impl<'d, ID, const AS: usize> BattleParty<'d, ID, AS> {
     pub fn know(&mut self, index: usize) -> Option<RemotePokemon> {
-        self.pokemon
-            .get_mut(index)
-            .map(BattlePokemon::know)
-            .flatten()
-    }
-
-    pub fn reveal_active(&mut self) {
-        for index in self.active.iter().flatten().map(PartyIndex::index) {
-            if let Some(pokemon) = self.pokemon.get_mut(index) {
-                pokemon.known = true;
-            }
-        }
+        self.pokemon.get_mut(index).map(HostPokemon::know).flatten()
     }
 
     pub fn ready_to_move(&self) -> bool {
         self.active
             .iter()
             .flatten()
-            .all(|a| a.queued_move.is_some())
+            .all(ActiveBattlePokemon::queued)
+            || self.active.iter().all(Option::is_none)
     }
 
     pub fn as_remote(&self) -> RemoteParty<ID, AS>
@@ -48,7 +35,7 @@ impl<'d, ID, const AS: usize> BattleParty<'d, ID, AS> {
                 .iter()
                 .map(|p| p.known.then(|| UnknownPokemon::new(p).uninit()))
                 .collect(),
-            active: ActivePokemon::into_remote(&self.active),
+            active: ActiveBattlePokemon::as_usize(&self.active),
         }
     }
 }
