@@ -1,13 +1,12 @@
 use serde::{Deserialize, Serialize};
 use serde_big_array::BigArray;
 
-use pokedex::pokemon::{party::Party, owned::SavedPokemon};
+use pokedex::pokemon::party::Party;
 
 use crate::{
-    message::{ClientMessage, ServerMessage},
-    party::{PartyIndex, PlayerParty, RemoteParty},
+    data::BattleData,
+    party::{Active, ActivePokemon, PlayerParty, RemoteParty},
     pokemon::PokemonView,
-    BattleData, BattleEndpoint,
 };
 
 #[derive(Debug, Clone, Copy, Deserialize, Serialize)]
@@ -21,16 +20,13 @@ impl Default for PlayerSettings {
     }
 }
 
-#[derive(Debug)]
-pub struct Player<ID, A: PartyIndex, P, E: BattleEndpoint<ID, AS>, const AS: usize> {
+pub struct Player<ID, A: ActivePokemon, P, E, const AS: usize> {
     pub party: PlayerParty<ID, A, P, AS>,
-    endpoint: E,
     pub settings: PlayerSettings,
+    pub endpoint: E,
 }
 
-impl<ID, A: PartyIndex, P: PokemonView, E: BattleEndpoint<ID, AS>, const AS: usize>
-    Player<ID, A, P, E, AS>
-{
+impl<ID, A: ActivePokemon, P: PokemonView, E, const AS: usize> Player<ID, A, P, E, AS> {
     pub fn new(
         id: ID,
         name: Option<String>,
@@ -52,31 +48,14 @@ impl<ID, A: PartyIndex, P: PokemonView, E: BattleEndpoint<ID, AS>, const AS: usi
     pub fn name(&self) -> &str {
         self.party.name()
     }
-
-    pub fn send(&mut self, message: ServerMessage<ID, AS>) {
-        self.endpoint.send(message)
-    }
-
-    pub fn receive(&mut self) -> Option<ClientMessage<ID>> {
-        self.endpoint.receive()
-    }
-}
-
-#[derive(Debug, Serialize, Deserialize)]
-pub struct LocalPlayer<ID, E: BattleEndpoint<ID, AS>, const AS: usize> {
-    pub id: ID,
-    pub name: Option<String>,
-    pub party: Party<SavedPokemon>,
-    pub settings: PlayerSettings,
-    pub endpoint: E,
 }
 
 #[derive(Debug, Clone, Deserialize, Serialize)]
-pub struct ValidatedPlayer<ID, const AS: usize> {
+pub struct ClientPlayerData<ID, const AS: usize> {
     pub id: ID,
     pub name: Option<String>,
     #[serde(with = "BigArray")]
-    pub active: [Option<usize>; AS],
+    pub active: Active<usize, AS>,
     pub data: BattleData,
     pub remotes: Vec<RemoteParty<ID, AS>>,
 }
